package metastudio.components.graphs;

import java.awt.Color;
import java.util.Iterator;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;

import metastudio.MultiBridge;
import metastudio.UserInterfacePanel;
import metastudio.data.graph.Graph;
import metastudio.data.graph.MetaGraphFactory;
import metastudio.data.graph.Node;
import metastudio.utils.Preferences;
import aterm.ATerm;
import aterm.ATermFactory;
import aterm.pure.PureFactory;

public class BoxTree extends UserInterfacePanel {
    private MetaGraphFactory factory;
    private JPanel rootPanel;

    public BoxTree(ATermFactory factory, MultiBridge bridge) {
        super(factory, bridge);
        this.factory = new MetaGraphFactory((PureFactory) factory);

        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

        rootPanel = new JPanel();
        rootPanel.setBackground(Color.white);
        JScrollPane scrolledPane = new JScrollPane(rootPanel);
        JViewport view = scrolledPane.getViewport();
        view.setBackground(Preferences.getColor("box.background"));
        add(scrolledPane);
    }

    public void displayGraph(String id, ATerm graphTerm) {
        if (id.equals(getId())) {
            Graph graph = factory.GraphFromTerm(graphTerm);
            setGraph(graph);
            repaint();
        }
    }

    private String getId() {
        return "parsetree";
    }

    private void setGraph(Graph graph) {
        rootPanel.removeAll();
        Node root = Graph.getRootNode(graph);

        if (root != null) {
            rootPanel.add(buildHierarchy(graph, root));
        }

        repaint();
    }

    // TODO: remove recursion here, it gets too deep!
    // Also: the graph should contain attributes telling what are characters, lists and normal nodes such
    // that this information can be used to make display decisions. Now it is reverse engineered from
    // the node shape what to do ;-(
    private JComponent buildHierarchy(Graph graph, Node node) {
        List children = Graph.getChildren(graph, node);

        if (children.size() == 0) {
        	if (Graph.getNodeShape(node).isEllipse()) {
              return new NodeBox(getBridge(), node, true);
        	}
        	else {
        		return null;
        	}
        } else {
            NodeBox box = new NodeBox(getBridge(), node, false);
            Iterator iter = children.iterator();

            while (iter.hasNext()) {
                Node child = (Node) iter.next();
                JComponent tree = buildHierarchy(graph, child);
                
                if (tree != null) {
                  box.add(tree);
                }
            }
            return box;
        }
    }
}
